from pydantic import Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.parts import ChannelSnippet
from youtube_pydantic_models.parts import ChannelContentDetails
from youtube_pydantic_models.parts import ChannelStatistics
from youtube_pydantic_models.parts import BaseTopicDetails
from youtube_pydantic_models.parts import ChannelStatus
from youtube_pydantic_models.parts import BaseBrandingSettings
from youtube_pydantic_models.parts import BaseAuditDetails
from youtube_pydantic_models.parts import BaseContentOwnerDetails
from youtube_pydantic_models.subparts import Localization


class YoutubeChannelResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: ChannelSnippet | None = Field(default=None)
    content_details: ChannelContentDetails | None = Field(default=None)
    statistics: ChannelStatistics | None = Field(default=None)
    topic_details: BaseTopicDetails | None = Field(default=None)
    status: ChannelStatus | None = Field(default=None)
    branding_settings: BaseBrandingSettings | None = Field(default=None)
    audit_details: BaseAuditDetails | None = Field(default=None)
    content_owner_details: BaseContentOwnerDetails | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)



from tests.youtube_client_for_testing import YoutubeClientForTesting


class TestGetVideo(YoutubeClientForTesting):
    def test_get_available_video(self):
        video_id = "PJm8WNajZtw"
        video = self.client.get_video(
            id=video_id,
            part="snippet, contentDetails, statistics, topicDetails, status, player, recordingDetails, localizations, liveStreamingDetails"
        )
        data = video.model_dump(
            by_alias=True,
            exclude_none=True
        )
        
        assert data['id'] == video_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert data['player'] is not None
        assert data['status'] is not None
    
    def test_get_unavailable_video(self):
        video_id = ""
        video = self.client.get_video(
            id=video_id,
            part="snippet, contentDetails, statistics, topicDetails, status, player, recordingDetails, localizations, liveStreamingDetails"
        )

        assert video is None
    
    def test_get_available_video_with_available_part(self):
        video_id = "PJm8WNajZtw"
        video = self.client.get_video(
            id=video_id,
            part="snippet, contentDetails, player, recordingDetails"
        )
        data = video.model_dump(
            by_alias=True,
            exclude_none=True
        )

        assert data['id'] == video_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert data['player'] is not None
        assert data['recordingDetails'] is not None
        assert len(data) == 7 # kind, etag, id, snippet, contentDetails, player, recordingDetails
    
    def test_get_available_video_with_unvailable_part(self):
        video_id = "PJm8WNajZtw"
        video = self.client.get_video(
            id=video_id,
            part="wrong, player, localizations"
        )

        assert video is None



from pydantic import Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.parts import SearchId
from youtube_pydantic_models.parts import SearchSnippet


class YoutubeSearchResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: SearchId | None = Field(default=None)
    snippet: SearchSnippet | None = Field(default=None)



from tests.youtube_client_for_testing import YoutubeClientForTesting


class TestGetPlaylist(YoutubeClientForTesting):
    def test_get_available_playlist(self):
        playlist_id = "PLOU2XLYxmsIL-LSajwT6_VviFWrDChEJX"
        playlist = self.client.get_playlist(
            id=playlist_id,
            part="snippet, contentDetails, player, status, localizations"
        )
        data = playlist.model_dump(
            by_alias=True,
            exclude_none=True
        )
        
        assert data['id'] == playlist_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert data['player'] is not None
        assert data['status'] is not None
    
    def test_get_unavailable_playlist(self):
        playlist_id = ""
        playlist = self.client.get_playlist(
            id=playlist_id,
            part="snippet, contentDetails, player, status, localizations"
        )

        assert playlist is None
    
    def test_get_available_playlist_with_available_part(self):
        playlist_id = "PLOU2XLYxmsIL-LSajwT6_VviFWrDChEJX"
        playlist = self.client.get_playlist(
            id=playlist_id,
            part="snippet, contentDetails, player"
        )
        data = playlist.model_dump(
            by_alias=True,
            exclude_none=True
        )

        assert data['id'] == playlist_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert data['player'] is not None
        assert len(data) == 6 # kind, etag, id, snippet, contentDetails, player
    
    def test_get_available_playlist_with_unvailable_part(self):
        playlist_id = "PLOU2XLYxmsIL-LSajwT6_VviFWrDChEJX"
        playlist = self.client.get_playlist(
            id=playlist_id,
            part="wrong, player, localizations"
        )

        assert playlist is None



class QuotaException(Exception):
    def __init__(self) -> None:
        super("unavailable quota")


class InvalidArgException(Exception):
    def __init__(self) -> None:
        super("invalid arg")


class RequiredArgException(Exception):
    def __init__(self, name: str) -> None:
        super(f"{name} arg is required")



import requests
from datetime import datetime
from enum import Enum
from youtube_pydantic_models.channel_resource import YoutubeChannelResource
from youtube_pydantic_models.playlist_resource import YoutubePlaylistResource
from youtube_pydantic_models.video_resource import YoutubeVideoResource
from youtube_pydantic_models.search_resource import YoutubeSearchResource
from youtube_pydantic_models.quotas import Quotas
from youtube_pydantic_models.errors import (
    InvalidArgException,
    RequiredArgException,
    QuotaException
)


class YoutubeClient:
    """
    YouTube's client based on requests.
    
    Attributes
    ----------
    _quota_per_day : int
        limit per dar of YouTube quota
    _current_quota : int
        current available quota
    _api_version : str
        version of YouTube API
    _api_service_name : str
        name of the service
    
    Methods
    -------
    get_api_service_name():
        Returns the value of _api_service_name private attribute.
    get_api_version():
        Returns the value of _api_version private attribute.
    get_quota_per_day():
        Returns the value of _quota_per_day private attribute.
    set_quota_per_day():
        Set a value of _quota_per_day private attribute.
    get_current_quota():
        Returns the value of _current_quota private attribute.
    get_channel(id: str, part: str):
        Returns a YoutubeChannelResource or None.
    get_playlist(id: str, part: str):
        Returns a YoutubePlaylistResource or None.
    get_video(id: str, part: str):
        Returns a YoutubeVideoResource or None.
    search(id: str, part: str, type: str):
        Returns a YoutubeSearchResource or None.
    """
    def __init__(
        self,
        api_key: str,
        quota_per_day: int = 10000,
        quotas_table: Enum = Quotas
    ) -> None:
        """
        Constructor.

        Parameters (required)
        ----------
            api_key : str
                secret YouTube API key
        
        Parameters (optional)
        ----------
            quota_per_day : int
                limit per day of YouTube quota. By default is 10000
            quotas_table : Enum
                table of quota's cost
        """
        if not api_key:
            raise RequiredArgException("api_key")
        
        self._quotas_table = quotas_table
        self._quota_per_day = quota_per_day
        self._current_quota = quota_per_day
        self._last_request = None
        self._api_key: str = api_key
        self._api_version: str = "v3"
        self._api_service_name: str = "youtube"
        self._api_base_path: str = f"https://www.googleapis.com/"
        self._api_path: str = f"{self._api_base_path}{self._api_service_name}/{self._api_version}/"

    def get_api_service_name(self) -> str:
        return self._api_service_name
    
    def get_api_version(self) -> str:
        return self._api_version
    
    def get_quota_per_day(self) -> int:
        return self._quota_per_day
    
    def set_quota_per_day(self, new_quota: int) -> None:
        if type(new_quota) != int or new_quota <= 0:
            raise InvalidArgException
        
        self._quota_per_day = new_quota
    
    def get_current_quota(self) -> int:
        return self._current_quota
    
    def get_channel(
        self,
        id: str,
        part: str
    ) -> YoutubeChannelResource | None:
        """
        Search a YouTube channel by id. 
        Select part of the resource.

        Parameters (required)
        ----------
            id : str
                channel id to search
            part : str
                resource's parts separated by comma (,). 
                Available parts: (snippet, contentDetails, statistics, 
                topicDetails, status, brandingSettings, contentOwnerDetails)
        
        Returns
        -------
        YoutubeChannelResource or None

        Errors
        ------
            QuotaException : unavailable YouTube quota
        """
        request_quota = self._quotas_table.list_channel.value
        if not self._check_available_quota(request_quota):
            raise QuotaException()
        
        params = {
            'id': id,
            'part': part
        }

        response = self._get_request(
            endpoint = "channels",
            params = params
        )

        self._sub_quota(request_quota)

        if not response:
            return None
        elif response['pageInfo']['totalResults'] > 0:
            data = response['items'][0]
            return YoutubeChannelResource(**data)
        return None

    def get_playlist(
        self,
        id: str,
        part: str
    ) -> YoutubePlaylistResource | None:
        """
        Search a YouTube playlist by id. 
        Select part of the resource.

        Parameters (required)
        ----------
            id : str
                playlist id to search
            part : str
                resource's parts separated by comma (,). 
                Available parts: (snippet, contentDetails, player, status, localizations)
        
        Returns
        -------
        YoutubePlaylistResource or None

        Errors
        ------
            QuotaException : unavailable YouTube quota
        """
        request_quota = self._quotas_table.list_playlist.value
        if not self._check_available_quota(request_quota):
            raise QuotaException()
        
        params = {
            'id': id,
            'part': part
        }

        response = self._get_request(
            endpoint = "playlists",
            params = params
        )

        self._sub_quota(request_quota)

        if not response:
            return None
        elif response['pageInfo']['totalResults'] > 0:
            data = response['items'][0]
            return YoutubePlaylistResource(**data)
        return None

    def get_video(
        self,
        id: str,
        part: str
    ) -> YoutubeVideoResource | None:
        """
        Search a YouTube video by id. 
        Select part of the resource.

        Parameters (required)
        ----------
            id : str
                video id to search
            part : str
                resource's parts separated by comma (,). 
                Available parts: (snippet, contentDetails, statistics, topicDetails, 
                status, player, recordingDetails, localizations, liveStreamingDetails)
        
        Returns
        -------
        YoutubeVideoResource or None

        Errors
        ------
            QuotaException : unavailable YouTube quota
        """
        request_quota = self._quotas_table.list_video.value
        if not self._check_available_quota(request_quota):
            raise QuotaException()
        
        params = {
            'id': id,
            'part': part
        }

        response = self._get_request(
            endpoint = "videos",
            params = params
        )

        self._sub_quota(request_quota)

        if not response:
            return None
        elif response['pageInfo']['totalResults'] > 0:
            data = response['items'][0]
            return YoutubeVideoResource(**data)
        return None

    def search(
        self,
        channel_id: str,
        part: str,
        type: str
    ) -> YoutubeSearchResource | None:
        """
        Search a YouTube resource by channel id. 
        Select part of the resource.

        Parameters (required)
        ----------
            channel_id : str
                channel id to search
            part : str
                resource's parts separated by comma (,). 
                Available parts: (snippet)
            type : str
                resource's type to filter.
                Available types: (channel, playlist, video)
        
        Returns
        -------
        YoutubeSearchResource or None

        Errors
        ------
            QuotaException : unavailable YouTube quota
        """
        request_quota = self._quotas_table.search.value
        if not self._check_available_quota(request_quota):
            raise QuotaException()
        
        params = {
            'channelId': channel_id,
            'part': part,
            'type': type
        }

        response = self._get_request(
            endpoint = "search",
            params = params
        )

        self._sub_quota(request_quota)

        if not response:
            return None
        elif response['pageInfo']['totalResults'] > 0:
            data = response['items'][0]
            return YoutubeSearchResource(**data)
        return None
    
    def _get_request(self, endpoint: str, params: dict) -> dict | list:
        response = requests.get(
            self._api_path + endpoint,
            params=self._set_key_param(params)
        )

        if response.status_code != 200:
            return None
        return response.json()
    
    def _set_key_param(self, params: dict) -> dict:
        params['key'] = self._api_key
        return params
    
    def _check_available_quota(self, next_quota: int) -> bool:
        now = datetime.now()
        if self._last_request and self._last_request.date() < now.date():
            self._current_quota = self._quota_per_day
        
        next_current_quota = self._current_quota - next_quota
        return self._quota_per_day >= next_current_quota
    
    def _sub_quota(self, sub: int) -> int:
        if type(sub) != int or sub <= 0:
            raise InvalidArgException
        
        self._last_request = datetime.now()
        self._current_quota -= sub
        return self._current_quota



from .parts import *
from .subparts import *
from .base_model_config import get_base_model_config
from .base_resource import YoutubeBaseResource
from .channel_resource import YoutubeChannelResource
from .errors import (
    QuotaException,
    RequiredArgException,
    InvalidArgException
)
from .list_response import YoutubeListResponse
from .main import YoutubeClient
from .playlist_resource import YoutubePlaylistResource
from .quotas import Quotas
from .search_resource import YoutubeSearchResource
from .video_resource import YoutubeVideoResource


from pydantic import Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.parts import VideoSnippet
from youtube_pydantic_models.parts import VideoContentDetails
from youtube_pydantic_models.parts import VideoStatus
from youtube_pydantic_models.parts import VideoStatistics
from youtube_pydantic_models.parts import VideoPlayer
from youtube_pydantic_models.parts import VideoTopicDetails
from youtube_pydantic_models.parts import BaseRecordingDetails
from youtube_pydantic_models.parts import BaseFileDetails
from youtube_pydantic_models.parts import BaseProcessingDetails
from youtube_pydantic_models.parts import BaseSuggestions
from youtube_pydantic_models.parts import BaseLiveStreamingDetails
from youtube_pydantic_models.subparts import Localization


class YoutubeVideoResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: VideoSnippet | None = Field(default=None)
    content_details: VideoContentDetails | None = Field(default=None)
    status: VideoStatus | None = Field(default=None)
    statistics: VideoStatistics | None = Field(default=None)
    player: VideoPlayer | None = Field(default=None)
    topic_details: VideoTopicDetails | None = Field(default=None)
    recording_details: BaseRecordingDetails | None = Field(default=None)
    file_details : BaseFileDetails | None = Field(default=None)
    processing_details: BaseProcessingDetails | None = Field(default=None)
    suggestions: BaseSuggestions | None = Field(default=None)
    live_streaming_details: BaseLiveStreamingDetails | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)



from pydantic import ConfigDict
from pydantic.alias_generators import to_camel


def get_base_model_config() -> ConfigDict:
    return ConfigDict(
        arbitrary_types_allowed = True,
        alias_generator = to_camel
        # extra = "forbid"
    )



from datetime import datetime
from pydantic import BaseModel, Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.subparts import (
    AudioStream,
    Channel,
    ContentRating,
    Image,
    Localized,
    ProcessingProgress,
    RegionRestriction,
    RelatedPlaylists,
    TagSuggestions,
    Thumbnail,
    VideoStream,
    Watch
)


class BaseAuditDetails(BaseModel):
    model_config = get_base_model_config()

    overall_good_standing: bool | None = Field(default=None)
    community_guide_lines_good_standing: bool | None = Field(default=None)
    copyright_strikes_good_standing: bool | None = Field(default=None)
    content_id_claims_good_standing: bool | None = Field(default=None)


class BaseBrandingSettings(BaseModel):
    model_config = get_base_model_config()

    channel: Channel | None = Field(default=None)
    image: Image | None = Field(default=None)
    watch: Watch | None = Field(default=None)


class ChannelContentDetails(BaseModel):
    model_config = get_base_model_config()

    related_playlists: RelatedPlaylists | None = Field(default=None)


class PlaylistContentDetails(BaseModel):
    model_config = get_base_model_config()

    item_count: int | None = Field(default=None)


class VideoContentDetails(BaseModel):
    model_config = get_base_model_config()

    duration: str | None = Field(default=None)
    dimension: str | None = Field(default=None)
    definition: str | None = Field(default=None)
    caption: str | None = Field(default=None)
    licensed_content: bool | None = Field(default=None)
    region_restriction: RegionRestriction | None = Field(default=None)
    content_rating: ContentRating | None = Field(default=None)
    projection: str | None = Field(default=None)
    has_custom_thumbnail: bool | None = Field(default=None)


class BaseContentOwnerDetails(BaseModel):
    model_config = get_base_model_config()

    content_owner: str | None = Field(default=None)
    time_linked: datetime | str | None = Field(default=None)


class BaseFileDetails(BaseModel):
    model_config = get_base_model_config()

    file_name: str | None = Field(default=None)
    file_size: int | None = Field(default=None)
    file_type: str | None = Field(default=None)
    container: str | None = Field(default=None)
    video_streams: list[VideoStream] | None = Field(default=None)
    audio_streams: list[AudioStream] | None = Field(default=None)
    duration_ms: int | None = Field(default=None)
    bitrate_bps: int | None = Field(default=None)
    creation_time: str | None = Field(default=None)


class SearchId(BaseModel):
    model_config = get_base_model_config()

    kind: str | None = Field(default=None)
    video_id: str | None = Field(default=None)
    channel_id: str | None = Field(default=None)
    playlist_id: str | None = Field(default=None)


class BaseLiveStreamingDetails(BaseModel):
    model_config = get_base_model_config()

    actual_start_time: datetime | str | None = Field(default=None)
    actual_end_time: datetime | str | None = Field(default=None)
    scheduled_start_time: datetime | str | None = Field(default=None)
    scheduled_end_time: datetime | str | None = Field(default=None)
    concurrent_viewers: int | None = Field(default=None)
    active_live_chat_id: str | None = Field(default=None)


class BasePageInfo(BaseModel):
    model_config = get_base_model_config()
    
    total_results: int | None = Field(default=None)
    results_per_page: int | None = Field(default=None)


class BasePlayer(BaseModel):
    model_config = get_base_model_config()

    embed_html: str | None = Field(default=None)


class VideoPlayer(BasePlayer):
    model_config = get_base_model_config()

    embed_height: int | None = Field(default=None)
    embed_width: int | None = Field(default=None)


class BaseProcessingDetails(BaseModel):
    model_config = get_base_model_config()

    processing_status: str | None = Field(default=None)
    processing_progress: ProcessingProgress | None = Field(default=None)
    processing_failure_reason: str | None = Field(default=None)
    file_details_availability: str | None = Field(default=None)
    processing_issues_availability: str | None = Field(default=None)
    tag_suggestions_availability: str | None = Field(default=None)
    editor_suggestions_availability: str | None = Field(default=None)
    thumbnails_availability: str | None = Field(default=None)


class BaseRecordingDetails(BaseModel):
    model_config = get_base_model_config()

    recording_date: datetime | str | None = Field(default=None)


class BaseSnippet(BaseModel):
    model_config = get_base_model_config()

    published_at: datetime | str | None = Field(default=None)
    title: str | None = Field(default=None)
    description: str | None = Field(default=None)
    thumbnails: dict[str, Thumbnail] | None = Field(default=None)


class SearchSnippet(BaseSnippet):
    model_config = get_base_model_config()

    channel_id: str | None = Field(default=None)
    channel_title: str | None = Field(default=None)
    live_broadcast_content: str | None = Field(default=None)
    publish_time: datetime | str | None = Field(default=None)


class ChannelSnippet(BaseSnippet):
    model_config = get_base_model_config()

    custom_url: str | None = Field(default=None)
    default_language: str | None = Field(default=None)
    localized: Localized | None = Field(default=None)
    country: str | None = Field(default=None)


class PlaylistSnippet(BaseSnippet):
    model_config = get_base_model_config()
    
    channel_id: str | None = Field(default=None)
    channel_title: str | None = Field(default=None)
    default_language: str | None = Field(default=None)
    localized: Localized | None = Field(default=None)


class VideoSnippet(PlaylistSnippet):
    model_config = get_base_model_config()

    tags: list[str] | None = Field(default=None)
    category_id: str | None = Field(default=None)
    live_broadcast_content: str | None = Field(default=None)
    default_audio_language: str | None = Field(default=None)


class BaseStatistics(BaseModel):
    model_config = get_base_model_config()

    view_count: str | None = Field(default=None)


class ChannelStatistics(BaseStatistics):
    model_config = get_base_model_config()

    subscriber_count: str | None = Field(default=None)
    hidden_subscriber_count: bool | None = Field(default=None)
    video_count: str | None = Field(default=None)


class VideoStatistics(BaseStatistics):
    model_config = get_base_model_config()

    like_count: str | None = Field(default=None)
    dislike_count: str | None = Field(default=None)
    favorite_count: str | None = Field(default=None)
    comment_count: str | None = Field(default=None)


class BaseStatus(BaseModel):
    model_config = get_base_model_config()

    privacy_status: str | None = Field(default=None)


class SharedStatus(BaseStatus):
    model_config = get_base_model_config()

    made_for_kids: bool | None = Field(default=None)
    self_declared_made_for_kids: bool | None = Field(default=None)


class ChannelStatus(SharedStatus):
    model_config = get_base_model_config()

    is_linked: bool | None = Field(default=None)
    long_uploads_status: str | None = Field(default=None)


class VideoStatus(SharedStatus):
    model_config = get_base_model_config()

    upload_status: str | None = Field(default=None)
    failure_reason: str | None = Field(default=None)
    rejection_reason: str | None = Field(default=None)
    publish_at: datetime | str | None = Field(default=None)
    license: str | None = Field(default=None)
    embeddable: bool | None = Field(default=None)
    public_stats_viewable: bool | None = Field(default=None)


class BaseSuggestions(BaseModel):
    model_config = get_base_model_config()

    processing_errors: list[str] | None = Field(default=None)
    processing_warnings: list[str] | None = Field(default=None)
    processing_hints: list[str] | None = Field(default=None)
    tag_suggestions: TagSuggestions | None = Field(default=None)
    editor_suggestions: list[str] | None = Field(default=None)


class BaseTopicDetails(BaseModel):
    model_config = get_base_model_config()

    topic_ids: list[str] | None = Field(default=None)
    topic_categories: list[str] | None = Field(default=None)


class VideoTopicDetails(BaseTopicDetails):
    model_config = get_base_model_config()

    relevant_topic_ids: list[str] | None = Field(default=None)



from tests.youtube_client_for_testing import YoutubeClientForTesting


class TestGetChannel(YoutubeClientForTesting):
    def test_get_available_channel(self):
        channel_id = "UC_x5XG1OV2P6uZZ5FSM9Ttw"
        channel = self.client.get_channel(
            id=channel_id,
            part="snippet, contentDetails, statistics, topicDetails, status, brandingSettings, contentOwnerDetails"
        )
        data = channel.model_dump(
            by_alias=True,
            exclude_none=True
        )

        assert data['id'] == channel_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert data['statistics'] is not None
        assert data['topicDetails'] is not None
        assert data['status'] is not None
        assert data['brandingSettings'] is not None
        assert data['contentOwnerDetails'] is not None
    
    def test_get_unavailable_channel(self):
        channel_id = ""
        channel = self.client.get_channel(
            id=channel_id,
            part="snippet, contentDetails, statistics, topicDetails, status, brandingSettings, contentOwnerDetails"
        )

        assert channel is None
    
    def test_get_available_channel_with_available_part(self):
        channel_id = "UC_x5XG1OV2P6uZZ5FSM9Ttw"
        channel = self.client.get_channel(
            id=channel_id,
            part="snippet, contentDetails"
        )
        data = channel.model_dump(
            by_alias=True,
            exclude_none=True
        )
        
        assert data['id'] == channel_id
        assert data['snippet'] is not None
        assert data['contentDetails'] is not None
        assert len(data) == 5 # kind, etag, id, snippet, contentDetails
    
    def test_get_available_channel_with_unvailable_part(self):
        channel_id = "UC_x5XG1OV2P6uZZ5FSM9Ttw"
        channel = self.client.get_channel(
            id=channel_id,
            part="wrongPart, snippet, contentDetails, statistics"
        )

        assert channel is None



from pydantic import BaseModel, Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)


class YoutubeBaseResource(BaseModel):
    model_config = get_base_model_config()

    kind: str | None = Field(default=None)
    etag: str | None = Field(default=None)



from pydantic import Field
from youtube_pydantic_models.base_model_config import (
    get_base_model_config
)
from youtube_pydantic_models.base_resource import (
    YoutubeBaseResource
)
from youtube_pydantic_models.parts import PlaylistSnippet
from youtube_pydantic_models.parts import BaseStatus
from youtube_pydantic_models.parts import PlaylistContentDetails
from youtube_pydantic_models.parts import BasePlayer
from youtube_pydantic_models.subparts import Localization


class YoutubePlaylistResource(YoutubeBaseResource):
    model_config = get_base_model_config()

    id: str | None = Field(default=None)
    snippet: PlaylistSnippet | None = Field(default=None)
    status: BaseStatus | None = Field(default=None)
    content_details: PlaylistContentDetails | None = Field(default=None)
    player: BasePlayer | None = Field(default=None)
    localizations: dict[str, Localization] | None = Field(default=None)
